from .gql_handler import *
from datetime import datetime
import asyncio

class PyXploraApi:
    def __init__(self, countryPhoneNumber: str, phoneNumber: str, password: str, userLang: str, timeZone: str, watchNo: int=0) -> None:
        self._countryPhoneNumber = countryPhoneNumber
        self._phoneNumber = phoneNumber
        self._password = password
        self._userLang = userLang
        self._timeZone = timeZone
        self._watchNo = watchNo
        self.__handler: GQLHandler = []
        self.__login()

    def __checkLogin(self) -> None:
        self.__handler = GQLHandler(self._countryPhoneNumber, self._phoneNumber, self._password, self._userLang, self._timeZone)
        self.__handler.login()
        #print("Login!")
        self.update()

    def __login(self) -> None:
        if not self.__handler:
            try:
                self.__checkLogin()
            except LoginError as error:
                #print(f"Error: -> First faill. {error.args[0]}")
                try:
                    self.__checkLogin()
                except LoginError as error:
                    #print(f"Error: -> Login canceled! {error.args[0]}")
                    raise Exception(f"{error.args[0]}")

    def update(self) -> None:
        self.__login()
        #print("update")

        self.watch_no = self._watchNo

        self.myInfo = self.__handler.getMyInfo()['readMyInfo']
        self.watch_user_id = self.myInfo['children'][self.watch_no]['ward']['id']
        self.watch_user_name = self.myInfo['children'][self.watch_no]['ward']['name']

        self.watch_last_location = self.__handler.getWatchLastLocation(self.watch_user_id)['watchLastLocate']

        self.contacts = []
        self.alarms = []
        self.chats = []
        self.safe_zones = []
        self.school_silent_mode = []

    def version(self) -> str:
        return "1.0.26"

##### Contact Info #####
    def getContacts(self) -> list:
        self.__login()
        for contact in self.__handler.getContacts(self.watch_user_id)['contacts']['contacts']:
            try:
                xcoin = contact['contactUser']['xcoin']
                id = contact['contactUser']['id']
            except KeyError and TypeError:
                xcoin = -1 # None - XCoins
                id = None
            self.contacts.append({
                'id': id,
                'guardianType': contact['guardianType'],
                'create': datetime.fromtimestamp(contact['create']).strftime('%Y-%m-%d %H:%M:%S'),
                'update': datetime.fromtimestamp(contact['update']).strftime('%Y-%m-%d %H:%M:%S'),
                'name': contact['name'],
                'phoneNumber': f"+{contact['countryPhoneNumber']}{contact['phoneNumber']}",
                'xcoin': xcoin,
            })
        return self.contacts

##### User Info #####
    def getUserID(self) -> str:
        return self.__checkMyInfo(self.myInfo, 'id', 'n/a')
    def getUserName(self) -> str:
        return self.__checkMyInfo(self.myInfo, 'name', 'n/a')
    def getUserIcon(self) -> str:
        return self.__checkMyInfo(self.__checkMyInfo(self.myInfo, 'extra', 'n/a'), 'profileIcon', 'n/a')
    def getUserXcoin(self) -> int:
        return self.__checkMyInfo(self.myInfo, 'xcoin', -1)
    def getUserCurrentStep(self) -> int:
        return self.__checkMyInfo(self.myInfo, 'currentStep', -1)
    def getUserTotalStep(self) -> int:
        return self.__checkMyInfo(self.myInfo, 'totalStep', -1)
    def getUserCreate(self) -> str:
        return datetime.fromtimestamp(self.__checkMyInfo(self.myInfo, 'create', 0)).strftime('%Y-%m-%d %H:%M:%S')
    def getUserUpdate(self) -> str:
        return datetime.fromtimestamp(self.__checkMyInfo(self.myInfo, 'update', 0)).strftime('%Y-%m-%d %H:%M:%S')

##### Watch Info #####
    def getWatchUserID(self) -> str:
        self.update()
        return self.watch_user_id
    def getWatchUserName(self) -> str:
        self.update()
        return self.watch_user_name
    def getWatchXcoin(self) -> int:
        self.update()
        return self.myInfo['children'][self.watch_no]['ward']['xcoin']
    def getWatchCurrentStep(self) -> int:
        self.update()
        return self.myInfo['children'][self.watch_no]['ward']['currentStep']
    def getWatchTotalStep(self) -> int:
        self.update()
        return self.myInfo['children'][self.watch_no]['ward']['totalStep']
    def getWatchAlarm(self) -> list:
        self.update()
        for alarm in self.__handler.getAlarms(self.watch_user_id)['alarms']:
            self.alarms.append({
                'name': alarm['name'],
                'start': alarm['start'],
                'weekRepeat': alarm['weekRepeat'],
                'status': alarm['status'],
            })
        return self.alarms
    def getWatchBattery(self) -> int:
        self.update()
        return self.watch_last_location['battery']
    async def __getWatchBattery_a(self) -> int:
        await self.__askWatchLocate_async()
        await asyncio.sleep(15)
        self.watch_last_location = (await self.__handler.getWatchLastLocation_a(self.watch_user_id))['watchLastLocate']
        if self.watch_last_location != None:
            if (int(datetime.timestamp(datetime.now())) - self.watch_last_location['tm']) < 30:
                return self.watch_last_location['battery']
        watch_location = await self.__askHelper()
        await asyncio.sleep(1)
        return watch_location['battery']
    def getWatchIsCharging(self) -> bool:
        self.update()
        try:
            return self.watch_last_location['isCharging']
        except TypeError:
            return False
    def getWatchOnlineStatus(self) -> WatchOnlineStatus:
        if self.askWatchLocate() == False:
            return WatchOnlineStatus.OFFLINE.value
        try:
            if self.trackWatchInterval() == -1:
                return WatchOnlineStatus.OFFLINE.value
            return self.__handler.getWatches(self.watch_user_id)['watches'][self.watch_no]['onlineStatus']
        except TypeError:
            return WatchOnlineStatus.UNKNOWN.value
    def getWatchUnReadChatMsgCount(self) -> int: # bug?
        self.update()
        return self.__handler.unReadChatMsgCount(self.watch_user_id)['unReadChatMsgCount']
    def getWatchChats(self) -> list: # bug?
        self.update()
        for chat in self.__handler.chats(self.watch_user_id)['chats']['list']:
            self.chats.append({
                'msgId': chat['msgId'],
                'type': chat['type'],
                #chat['sender'],
                'sender_id': chat['sender']['id'],
                'sender_name': chat['sender']['name'],
                #chat['receiver'],
                'receiver_id': chat['receiver']['id'],
                'receiver_name': chat['receiver']['name'],
                #chat['data'],
                'data_text': chat['data']['text'],
                'data_sender_name': chat['data']['sender_name'],
            })
        return self.chats

##### Watch Location Info #####
    def getWatchLastLocation(self) -> dict:
        self.update()
        return self.watch_last_location
    def getWatchLocateType(self) -> str:
        self.update()
        return self.watch_last_location['locateType']
    def getWatchLocate(self) -> dict:
        self.update()
        return {
            'tm': datetime.fromtimestamp(self.watch_last_location['tm']).strftime('%Y-%m-%d %H:%M:%S'),
            'lat': self.watch_last_location['lat'],
            'lng': self.watch_last_location['lng'],
            'rad': self.watch_last_location['rad'],
            'poi': self.watch_last_location['poi'],
            'city': self.watch_last_location['city'],
            'province': self.watch_last_location['province'],
            'country': self.watch_last_location['country'],
        }
    def getWatchIsInSafeZone(self) -> bool:
        self.update()
        return self.watch_last_location['isInSafeZone']
    def getWatchSafeZoneLabel(self) -> str:
        self.update()
        return self.watch_last_location['safeZoneLabel']
    def getSafeZones(self) -> list:
        self.update()
        for safeZone in self.__handler.safeZones(self.watch_user_id)['safeZones']:
            self.safe_zones.append({
                #safeZone,
                'groupName': safeZone['groupName'],
                'name': safeZone['name'],
                'lat': safeZone['lat'],
                'lng': safeZone['lng'],
                'rad': safeZone['rad'],
                'address': safeZone['address'],
            })
        return self.safe_zones
    def trackWatchInterval(self) -> int:
        self.update()
        return self.__handler.trackWatch(self.watch_user_id)['trackWatch']
    def askWatchLocate(self) -> bool:
        self.update()
        return self.__handler.askWatchLocate(self.watch_user_id)['askWatchLocate']
    async def __askWatchLocate_async(self) -> bool:
        self.update()
        return (await self.__handler.askWatchLocate_a(self.watch_user_id))['askWatchLocate']

##### Feature #####
    def schoolSilentMode(self) -> list:
        self.update()
        for sientTime in self.__handler.silentTimes(self.watch_user_id)['silentTimes']:
            self.school_silent_mode.append({
                'id': sientTime['id'],
                'start': self.__helperTime(sientTime['start']),
                'end': self.__helperTime(sientTime['end']),
                'weekRepeat': sientTime['weekRepeat'],
                'status': sientTime['status'],
            })
        return self.school_silent_mode
    def setEnableSilentTime(self, silentId) -> bool:
        self.update()
        return bool(self.__handler.setEnableSlientTime(silentId)['setEnableSilentTime'])
    def setDisableSilentTime(self, silentId) -> bool:
        self.update()
        return bool(self.__handler.setEnableSlientTime(silentId, NormalStatus.DISABLE)['setEnableSilentTime'])
    def setAllEnableSilentTime(self) -> list:
        res = []
        for silentTime in self.schoolSilentMode():
            res.append(bool(self.setEnableSilentTime(silentTime['id'])))
        return res
    def setAllDisableSilentTime(self) -> list:
        res = []
        for silentTime in self.schoolSilentMode():
            res.append(bool(self.setDisableSilentTime(silentTime['id'])))
        return res
    def sendText(self, text): # sender is login User
        self.update()
        return self.__handler.sendText(self.watch_user_id, text)
    def shutdown(self) -> bool:
        self.update()
        return self.__handler.shutdown(self.watch_user_id)
    def reboot(self) -> bool:
        self.update()
        return self.__handler.reboot(self.watch_user_id)

##### - #####
    def __helperTime(self, time) -> str:
        h = str(int(time) /60).split('.')
        h2 = str(int(h[1]) *60).zfill(2)[:2]
        return h[0].zfill(2) + ":" + str(h2).zfill(2)
    async def __askHelper(self) -> int:
        try:
            return (await self.__handler.getWatchLastLocation_a(self.watch_user_id))['watchLastLocate']
        except TypeError:
            return 0
    def __checkMyInfo(self, func, value: str, res):
        if value in func:
            return func[value]
        return res



MUTATION = {
    "tokenM": "mutation IssueToken($countryPhoneNumber: String!, $phoneNumber: String!, $password: String!, $userLang: String!, $timeZone: String!) {\n  issueToken(countryPhoneNumber: $countryPhoneNumber, phoneNumber: $phoneNumber, password: $password, userLang: $userLang, timeZone: $timeZone) {\n    __typename\n    id\n    token\n    issueDate\n    expireDate\n    user {\n      __typename\n      ...UserFragment\n    }\n    app {\n      __typename\n      ...AppFragment\n    }\n    valid\n    w360 {\n      __typename\n      token\n      secret\n      qid\n    }\n  }\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment AppFragment on App {\n  __typename\n  id\n  name\n  packageName\n  apiKey\n  apiSecret\n  terminalType\n  description\n  status\n  versions {\n    __typename\n    ...VersionFragment\n  }\n  create\n  update\n}\nfragment VersionFragment on AppVersion {\n  __typename\n  id\n  version\n  requireUpdate\n  downloadUrl\n  description\n  create\n  update\n}",
    "sendTextM": "mutation SendChatText($uid : String!, $text : String!) {\n  sendChatText(uid: $uid, text: $text)\n}",
    "modifyAlertM": "mutation modifyAlert($uid: String!, $remind: YesOrNo!) {\n  modifyAlert(uid: $uid, remind: $remind)\n}",
    "shutdownM": "mutation ShutDown($uid: String!) {\n  shutDown(uid: $uid)\n}",
    "rebootM": "mutation reboot($uid : String!) {\n  reboot(uid: $uid)\n}",
    "addStepM": "mutation AddStep($stepCount: Int) {\n  addStep(stepCount: $stepCount)\n}",
    "setEnableSlientTimeM": "mutation SetEnableSlientTime($silentId : String!, $status: NormalStatus!) {\n  setEnableSilentTime(silentId: $silentId, status: $status)\n}",
}


QUERY = {
    "alarmsQ": "query Alarms($uid: String!) {\n  alarms(uid: $uid) {\n    __typename\n    ...WatchAlarmFragment\n  }\n}\nfragment WatchAlarmFragment on WatchAlarm {\n  __typename\n  id\n  vendorName\n  vendorId\n  watch {\n    __typename\n    ...WatchFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  start\n  end\n  weekRepeat\n  occurMin\n  description\n  extra\n  start\n  create\n  update\n  status\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "cardGroupsQ": "query CardGroups($isCampaign : Boolean!) {\n  cardGroups(status: ENABLE, isCampaign: $isCampaign) {\n    __typename\n    id\n    name\n    type\n    kind\n    status\n    listOrder\n    extra\n    description\n    create\n    update\n  }\n}",
    "contactsQ": "query Contacts($uid: String) {\n  contacts(uid: $uid) {\n    __typename\n    ...ContactListFragment\n  }\n}\nfragment ContactListFragment on XPContactList {\n  __typename\n  contacts {\n    __typename\n    ...ContactFragment\n  }\n  followRequest {\n    __typename\n    ...FollowRequestFragment\n  }\n}\nfragment ContactFragment on XPContact {\n  __typename\n  id\n  ownUser {\n    __typename\n    ...SimpleUserFragment\n  }\n  name\n  countryPhoneNumber\n  phoneNumber\n  contactUser {\n    __typename\n    ...SimpleUserFragment\n  }\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  approval\n  guardianType\n  create\n  update\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment FollowRequestFragment on XPFollowRequest {\n  __typename\n  id\n  qid\n  deviceId\n  name\n  iconUrl\n  phoneNumber\n  countryCode\n  requestTime\n}",
    "readMyInfoQ": "query ReadMyInfo {\n  readMyInfo {\n    __typename\n    ...UserFragment\n  }\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "watchesQ": "query Watches($uid: String) {\n  watches(uid: $uid) {\n    __typename\n    ...WatchFragment\n  }\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "watchLastLocateQ": "query WatchLastLocate($uid: String!) {\n  watchLastLocate(uid: $uid) {\n    __typename\n    ...WatchLastLocateFragment\n  }\n}\nfragment WatchLastLocateFragment on Location {\n  __typename\n  tm\n  lat\n  lng\n  rad\n  country\n  countryAbbr\n  province\n  city\n  addr\n  poi\n  battery\n  isCharging\n  isAdjusted\n  locateType\n  step\n  distance\n  isInSafeZone\n  safeZoneLabel\n}",
    "askWatchLocateQ": "query AskWatchLocate($uid: String!) {\n  askWatchLocate(uid: $uid)\n}",
    "trackWatchQ": "query TrackWatch($uid: String!) {\n  trackWatch(uid: $uid)\n}",
    "watchCountQ": "query FollowRequestWatchCount {\n  followRequestWatchCount\n}",
    "getWifiQ": "query GetWifis($uid : String!) {\n  getWiFis(uid: $uid) {\n    __typename\n    ...WiFiFragment\n  }\n}\nfragment WiFiFragment on WiFi {\n  __typename\n  id\n  name\n  ssid\n  password\n  state\n  create\n  update\n  user {\n    __typename\n    ...UserFragment\n  }\n  watch {\n    __typename\n    ...WatchFragment\n  }\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}",
    "avatarsQ": "query Avatars($id: String) {\n  avatars(id: $id) {\n    __typename\n    id\n    name\n    file {\n      __typename\n      ...FileFragment\n    }\n    listOrder\n    create\n    update\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}",
    "subscribedCampaignQ": "query SubscribedCampaign($uid: String!, $needDetail: Boolean!) {\n  subscribedCampaign(uid:$uid) {\n    __typename\n    campaign @include(if: $needDetail) {\n      __typename\n      ...CampaignFragment\n    }\n    user @include(if: $needDetail) {\n      __typename\n      ...UserFragment\n    }\n    avatar @include(if: $needDetail) {\n      __typename\n      ...FileFragment\n    }\n    profile @include(if: $needDetail) {\n      __typename\n      ...FileFragment\n    }\n    name\n    subscribedDate\n  }\n}\nfragment CampaignFragment on Campaign {\n  __typename\n  id\n  category {\n    __typename\n    ...CampaignCategoryFragment\n  }\n  name {\n    __typename\n    ...MultiLangTextFragment\n  }\n  lgProfile {\n    __typename\n    ...FileFragment\n  }\n  cardProfile {\n    __typename\n    ...FileFragment\n  }\n  cardMov {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  description {\n    __typename\n    ...MultiLangTextFragment\n  }\n  content {\n    __typename\n    ...MultiLangTextFragment\n  }\n  rewardDescription {\n    __typename\n    ...MultiLangTextFragment\n  }\n  tags {\n    __typename\n    ...CampaignTagFragmet\n  }\n  extra\n  start\n  end\n  status\n  new\n  hot\n  create\n  update\n}\nfragment CampaignCategoryFragment on CampaignCategory {\n  __typename\n  id\n  name {\n    __typename\n    ...MultiLangTextFragment\n  }\n  status\n  create\n  update\n  file {\n    __typename\n    ...FileFragment\n  }\n}\nfragment MultiLangTextFragment on MultiLangText {\n  __typename\n  id\n  key\n  langId\n  langKey\n  text\n  type\n  create\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment CampaignTagFragmet on CampaignTag {\n  __typename\n  id\n  name {\n    __typename\n    ...MultiLangTextFragment\n  }\n  create\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "staticCardQ": "query StaticCard {\n  staticCards(groupId: \"#groupId\") {\n    __typename\n    id\n    title {\n      __typename\n      ...MultiLangTextFragment\n    }\n    type\n    lgBackground {\n      __typename\n      ...FileFragment\n    }\n    smBackground {\n      __typename\n      ...FileFragment\n    }\n    mov {\n      __typename\n      ...FileFragment\n    }\n    bgColor\n    fgColor\n    extra\n    description {\n      __typename\n      ...MultiLangTextFragment\n    }\n    status\n    width\n    height\n    create\n    update\n    campaign {\n      __typename\n      ...CampaignFragment\n    }\n    docLink {\n      __typename\n      ...DocLinkFragment\n    }\n    campaignRanks {\n      __typename\n      ...RankCardFragment\n    }\n    product {\n      __typename\n      ...UnpurchaseProductFragment\n    }\n  }\n}\nfragment MultiLangTextFragment on MultiLangText {\n  __typename\n  id\n  key\n  langId\n  langKey\n  text\n  type\n  create\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment CampaignFragment on Campaign {\n  __typename\n  id\n  category {\n    __typename\n    ...CampaignCategoryFragment\n  }\n  name {\n    __typename\n    ...MultiLangTextFragment\n  }\n  lgProfile {\n    __typename\n    ...FileFragment\n  }\n  cardProfile {\n    __typename\n    ...FileFragment\n  }\n  cardMov {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  description {\n    __typename\n    ...MultiLangTextFragment\n  }\n  content {\n    __typename\n    ...MultiLangTextFragment\n  }\n  rewardDescription {\n    __typename\n    ...MultiLangTextFragment\n  }\n  tags {\n    __typename\n    ...CampaignTagFragmet\n  }\n  extra\n  start\n  end\n  status\n  new\n  hot\n  create\n  update\n}\nfragment CampaignCategoryFragment on CampaignCategory {\n  __typename\n  id\n  name {\n    __typename\n    ...MultiLangTextFragment\n  }\n  status\n  create\n  update\n  file {\n    __typename\n    ...FileFragment\n  }\n}\nfragment CampaignTagFragmet on CampaignTag {\n  __typename\n  id\n  name {\n    __typename\n    ...MultiLangTextFragment\n  }\n  create\n}\nfragment DocLinkFragment on DocLink {\n  __typename\n  id\n  campaignId\n  title {\n    __typename\n    ...MultiLangTextFragment\n  }\n  link\n  type\n  lgImage {\n    __typename\n    ...FileFragment\n  }\n  status\n  description {\n    __typename\n    ...MultiLangTextFragment\n  }\n  create\n  update\n}\nfragment RankCardFragment on RankCard {\n  __typename\n  campaign {\n    __typename\n    ...CampaignFragment\n  }\n  topRank {\n    __typename\n    ...RankRowFragment\n  }\n  userRank {\n    __typename\n    ...RankRowFragment\n  }\n}\nfragment RankRowFragment on RankRow {\n  __typename\n  user {\n    __typename\n    ...SimpleUserFragment\n  }\n  score\n  rank\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment UnpurchaseProductFragment on UnpurchaseProduct {\n  __typename\n  productId\n  productName\n}",
    "CampaignUserProfilesQ": "query CampaignUserProfiles {\n  campaignUserProfiles {\n    __typename\n    ...CampaignUserProfilesFragment\n  }\n  followRequestWatchCount\n}\nfragment CampaignUserProfilesFragment on CampaignUserProfile {\n  __typename\n  id\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  avatar {\n    __typename\n    ...AvatarFragment\n  }\n  profile {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment AvatarFragment on Avatar {\n  __typename\n  id\n  name\n  file {\n    __typename\n    ...FileFragment\n  }\n}",
    "GetReviewStatusQ": "query GetReviewStatus($uid: String!) {\n  getReviewStatus(uid: $uid)\n}",
    "CountriesQ": "query Countries {\n  countries(status: ENABLE) {\n    __typename\n    ...CountryFragment\n  }\n}\nfragment CountryFragment on Country {\n  __typename\n  id\n  code\n  name\n  status\n  listOrder\n  create\n}",
    "unReadChatMsgCountQ": "query UnReadChatMsgCount($uid : String!) {\n  unReadChatMsgCount(uid: $uid)\n}",
    "noticeQ": "query noticeQ {\n  getNotices {\n    id\n    noticeType\n    name\n    desc\n    content\n    startDate\n    endDate\n    status\n    cDate\n    uDate\n  }\n}",
    "chatsQ": "query Chats($uid : String!, $offset: Int, $limit: Int, $msgId: String) {\n  chats(uid: $uid, offset: $offset, limit: $limit, msgId: $msgId) {\n    __typename\n    offset\n    limit\n    list {\n      __typename\n      ...SimpleChatFragment\n    }\n  }\n}\nfragment SimpleChatFragment on SimpleChat {\n  __typename\n  id\n  msgId\n  type\n  sender {\n    __typename\n    ...SimpleUserFragment\n  }\n  receiver {\n    __typename\n    ...SimpleUserFragment\n  }\n  data\n  create\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "safeZonesQ": "query SafeZones($uid: String!) {\n  safeZones(uid: $uid) {\n    __typename\n    ...SafeZoneFragment\n  }\n}\nfragment SafeZoneFragment on SafeZone {\n  __typename\n  id\n  vendorName\n  vendorId\n  groupName\n  watch {\n    __typename\n    ...WatchFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  group {\n    __typename\n    ...SafeZoneGroupFragment\n  }\n  name\n  lat\n  lng\n  rad\n  address\n  description\n  extra\n  create\n  update\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment SafeZoneGroupFragment on SafeZoneGroup {\n  __typename\n  id\n  name\n  status\n  description\n  create\n  update\n}",
    "safeZoneGroupsQ": "query SafeZoneGroups {\n  safeZoneGroups(status: ENABLE) {\n    __typename\n    ...SafeZoneGroupFragment\n  }\n}\nfragment SafeZoneGroupFragment on SafeZoneGroup {\n  __typename\n  id\n  name\n  status\n  description\n  create\n  update\n}",
    "silentTimesQ": "query SlientTimes($uid : String!) {\n  silentTimes(uid: $uid) {\n    __typename\n    ...SilentModeFragment\n  }\n}\nfragment SilentModeFragment on SilentMode {\n  __typename\n  id\n  vendorName\n  vendorId\n  watch {\n    __typename\n    ...WatchFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  start\n  end\n  weekRepeat\n  description\n  extra\n  status\n  create\n  update\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
}


import hashlib
import math
import time
from datetime import date, datetime, timezone
from python_graphql_client import GraphqlClient

from enum import Enum

from . import gql_mutations as gm
from . import gql_queries as gq

class NormalStatus(Enum):
    ENABLE = "ENABLE"
    DISABLE = "DISABLE"
    UNKNOWN__ = "UNKNOWN__"

class WatchOnlineStatus(Enum):
    UNKNOWN = "UNKNOWN"
    ONLINE = "ONLINE"
    OFFLINE = "OFFLINE"
    UNKNOWN__ = "UNKNOWN__"

class GQLHandler:
    def __init__(self, countryPhoneNumber: str, phoneNumber: str, password: str, userLang: str, timeZone: str):
        # init vars
        self.sessionId = None
        self.accessToken = None
        self.accessTokenExpire = 0
        self.userLocale = userLang
        self.timeZone = timeZone
        self.countryPhoneNumber = countryPhoneNumber
        self.phoneNumber = phoneNumber
        self.passwordMD5 = hashlib.md5(password.encode()).hexdigest()
        self.API_KEY = "270072d0fb4811ebacd96f6726fbdbb1"
        self.API_SECRET = "2d0288d0fb4811ebabfbd57e57c6ae64"
        self.issueDate = 0
        self.expireDate = 0
        self.userId = None
        self.ENDPOINT = "https://api.myxplora.com/api"
        self.variables = {
            "countryPhoneNumber": self.countryPhoneNumber,
            "phoneNumber": self.phoneNumber,
            "password": self.passwordMD5,
            "userLang": self.userLocale,
            "timeZone": self.timeZone
        }

        self.issueToken = None

    def getRequestHeaders(self, acceptedContentType: str):
        if acceptedContentType == "" or acceptedContentType == None:
            raise Exception("acceptedContentType MUST NOT be empty!")
        if self.API_KEY == None:
            raise Exception("XploraO2O API_KEY MUST NOT be empty!")
        if self.API_SECRET == None:
            raise Exception("XploraO2O API_SECRET MUST NOT be empty!")
        requestHeaders = {}
        if self.accessToken == None:
            # OPEN authorization
            authorizationHeader = f"Open {self.API_KEY}:{self.API_SECRET}"
        else:
            # BEARER authorization
            authorizationHeader = f"Bearer {self.accessToken}:{self.API_SECRET}"
            rfc1123DateString = datetime.now(timezone.utc).strftime("%a, %d %b %Y %H:%M:%S") + " GMT"
            requestHeaders["H-Date"] = rfc1123DateString
            requestHeaders["H-Authorization"] = authorizationHeader
        requestHeaders["H-BackDoor-Authorization"] = authorizationHeader
        requestHeaders["Accept"] = acceptedContentType
        requestHeaders["Content-Type"] = acceptedContentType
        requestHeaders["H-Tid"] = str(math.floor(time.time()))
        return requestHeaders

    def runGqlQuery(self, query: str, variables):
        if query == None:
            raise Exception("GraphQL guery string MUST NOT be empty!")
        # Add Xplora API headers
        requestHeaders = self.getRequestHeaders("application/json; charset=UTF-8")
        # create GQLClient
        gqlClient = GraphqlClient(endpoint=self.ENDPOINT, headers=requestHeaders)
        # execute QUERY|MUTATION
        data = gqlClient.execute(query=query, variables=variables)
        return data
    async def runGqlQuery_a(self, query: str, variables):
        if query == None:
            raise Exception("GraphQL guery string MUST NOT be empty!")
        # Add Xplora API headers
        requestHeaders = self.getRequestHeaders("application/json; charset=UTF-8")
        # create GQLClient
        gqlClient = GraphqlClient(endpoint=self.ENDPOINT, headers=requestHeaders)
        # execute QUERY|MUTATION
        data = await gqlClient.execute_async(query=query, variables=variables)
        return data

    def runAuthorizedGqlQuery(self, query: str, variables):
        if self.accessToken == None:
            raise Exception("You have to login to the Xplora API first.")
        # Run GraphQL query and return
        return self.runGqlQuery(query, variables)
    async def runAuthorizedGqlQuery_a(self, query: str, variables):
        if self.accessToken == None:
            raise Exception("You have to login to the Xplora API first.")
        # Run GraphQL query and return
        return await self.runGqlQuery_a(query, variables)

    def login(self):
        data = self.runGqlQuery(gm.MUTATION["tokenM"], self.variables)['data']
        if data['issueToken'] == None:
            # Login failed.
            raise LoginError("Login to Xplora API failed. Check your input!")
        self.issueToken = data['issueToken']

        #  Login succeeded
        self.sessionId = self.issueToken['id'];
        self.userId = self.issueToken['user']['id'];
        self.accessToken = self.issueToken['token'];
        self.issueDate = self.issueToken['issueDate'];
        self.expireDate = self.issueToken['expireDate'];
    
        if self.issueToken['app'] != None:
            #  Update API_KEY and API_SECRET?
            if self.issueToken['app']['apiKey']:
                self.API_KEY = self.issueToken['app']['apiKey']
            if self.issueToken['app']['apiSecret']:
                self.API_SECRET = self.issueToken['app']['apiSecret']
        return self.issueToken

    def isLogged(self):
        if self.issueToken:
            return True
        else: # login failed
            raise LoginError("Login to Xplora API failed.", 2)

    def isAdmin(self, ownId, query, variables, key):
        contacts = self.getContacts(ownId)
        for contact in contacts['contacts']['contacts']:
            try:
                id = contact['contactUser']['id']
            except KeyError and TypeError:
                id = None
            if self.userId == id:
                if contact['guardianType'] == 'FIRST':
                    return self.runAuthorizedGqlQuery(query, variables)['data'][key]
        return False
        #raise Exception("no Admin!")

########## SECTION QUERY start ##########

    def getMyInfo(self): # Profil from login Account
        self.isLogged()
        return self.runAuthorizedGqlQuery(gq.QUERY['readMyInfoQ'], {})['data']

    def getContacts(self, ownId): # Contacts from ownUser
        return self.runAuthorizedGqlQuery(gq.QUERY['contactsQ'], { "uid": ownId })['data']

    def getWatchCount(self):
        return self.runAuthorizedGqlQuery(gq.QUERY['watchCountQ'], {})['data']

    def getWatches(self, ownId):
        return self.runAuthorizedGqlQuery(gq.QUERY['watchesQ'], { "uid": ownId })['data']

    def getWatchLastLocation(self, ownId):
        return self.runAuthorizedGqlQuery(gq.QUERY['watchLastLocateQ'], { "uid": ownId })['data']

    async def getWatchLastLocation_a(self, ownId):
        return (await self.runAuthorizedGqlQuery_a(gq.QUERY['watchLastLocateQ'], { "uid": ownId }))['data']

    def trackWatch(self, ownId): # tracking time - seconds
        res = self.runAuthorizedGqlQuery(gq.QUERY['trackWatchQ'], { "uid": ownId })['data']
        if res['trackWatch'] != None:
            return res
        return { 'trackWatch': -1 }

    def askWatchLocate(self, ownId):
        res = self.runAuthorizedGqlQuery(gq.QUERY['askWatchLocateQ'], { "uid": ownId })['data']
        if res['askWatchLocate'] != None:
            return res
        return { 'askWatchLocate': False }

    async def askWatchLocate_a(self, ownId):
        res = (await self.runAuthorizedGqlQuery_a(gq.QUERY['askWatchLocateQ'], { "uid": ownId }))['data']
        if res['askWatchLocate'] != None:
            return res
        return { 'askWatchLocate': False }

    def getAlarms(self, ownId):
        return self.runAuthorizedGqlQuery(gq.QUERY['alarmsQ'], { "uid": ownId })['data']

    def getWifi(self, id): # without function?
        return self.runAuthorizedGqlQuery(gq.QUERY['getWifiQ'], { "uid": id })['data']

    def avatars(self, id): # without function?
        return self.runAuthorizedGqlQuery(gq.QUERY['avatarsQ'], { 'id': id })['data']

    def unReadChatMsgCount(self, ownId):
        return self.runAuthorizedGqlQuery(gq.QUERY['unReadChatMsgCountQ'], { 'uid': ownId })['data']

    def chats(self, ownId): # ownUser id
        return self.runAuthorizedGqlQuery(gq.QUERY['chatsQ'], { 'uid': ownId })['data']

    def notice(self): # without function?
        return self.runAuthorizedGqlQuery(gq.QUERY['noticeQ'], {})['data']

    def staticCard(self):
        return self.runAuthorizedGqlQuery(gq.QUERY['staticCardQ'], {})['data']

    def campaignUserProfiles(self):
        return self.runAuthorizedGqlQuery(gq.QUERY['CampaignUserProfilesQ'], {})['data']

    def subscribedCampaign(self, id, needDetail=False):
        return self.runAuthorizedGqlQuery(gq.QUERY['subscribedCampaignQ'], { 'uid': id, 'needDetail': needDetail })['data']

    def getReviewStatus(self, id):
        return self.runAuthorizedGqlQuery(gq.QUERY['GetReviewStatusQ'], { 'uid': id })['data']

    def countries(self): # Country Support
        return self.runAuthorizedGqlQuery(gq.QUERY['CountriesQ'], {})['data']

    def safeZones(self, ownId):
        return self.runAuthorizedGqlQuery(gq.QUERY['safeZonesQ'], { 'uid': ownId })['data']

    def safeZoneGroups(self):
        return self.runAuthorizedGqlQuery(gq.QUERY['safeZoneGroupsQ'], {})['data']

    def silentTimes(self, ownId):
        return self.runAuthorizedGqlQuery(gq.QUERY['silentTimesQ'], { 'uid': ownId })['data']

########## SECTION QUERY end ##########
    
########## SECTION MUTATION start ##########

    def sendText(self, ownId, text): # ownUser id
        if self.runAuthorizedGqlQuery(gm.MUTATION['sendTextM'], { "uid": ownId, "text": text })['data']['sendChatText'] != None:
            return True
        return False

    def addStep(self, stepCount):
        return self.runAuthorizedGqlQuery(gm.MUTATION['addStepM'], { "stepCount": stepCount })['data']

    def shutdown(self, ownId): # ownUser id
        return self.isAdmin(ownId, gm.MUTATION['shutdownM'], { "uid": ownId }, 'reboot')

    def reboot(self, ownId): # ownUser id
        return self.isAdmin(ownId, gm.MUTATION['rebootM'], { "uid": ownId }, 'reboot')

    def modifyAlert(self, id, YesOrNo): # function?
        return self.runAuthorizedGqlQuery(gm.MUTATION['modifyAlertM'], { "uid": id, "remind": YesOrNo })

    def setEnableSlientTime(self, silentId, status: NormalStatus=NormalStatus.ENABLE):
        return self.runAuthorizedGqlQuery(gm.MUTATION['setEnableSlientTimeM'], { 'silentId': silentId, 'status': status.value })['data']

########## SECTION MUTATION end ##########

class Error(Exception):
    pass

class LoginError(Error):
    def __init__(self, message, res=1):
        self.message = message
        self.res = res
        super().__init__(self.message, self.res)

    def __str__(self):
        return f'{self.message} - {self.res}'


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyxplora_api",
    version="1.0.26",
    author="Ludy87",
    author_email="android@astra-g.org",
    description="Python Xplora-Api",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Ludy87/pyxplora_api",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "Topic :: Utilities",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="api xplora",
    install_requires=["python-graphql-client==0.4.3"],
)